/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package html.shared.drawing;

import static html.shared.angles.Angles.FindAngle;
import static html.shared.points.Points.MaxDistanceEndPoint;
import static html.shared.rounding.Rounding.fixDecimal;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.util.ArrayList;
import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

/**
 *
 * @author Paul Long
 * 
 * Drawing helper class
 * Draws BufferedImages used in html output
 */
public class Drawing
{
    public static float ConvertToRadians(float angle)
    {
	return (float)(angle / 180 * Math.PI);
    }

    public static void drawTriangle1(Graphics gr, float angles[], Point ptA, Point ptB, Point ptC, JTextField a, JTextField b, JTextField c, String strA, String strB, String strC)
    {
        
        gr.setColor(Color.white);
        gr.fillRect(0, 0, 400, 300);
        gr.setColor(Color.black);
        gr.drawLine((int)ptA.getX(), (int)ptA.getY(), (int)ptB.getX(), (int)ptB.getY());
        gr.drawLine((int)ptB.getX(), (int)ptB.getY(), (int)ptC.getX(), (int)ptC.getY());
        gr.drawLine((int)ptC.getX(), (int)ptC.getY(), (int)ptA.getX(), (int)ptA.getY());
        
        AffineTransform at = ((Graphics2D)gr).getTransform();
        
        gr.translate((int)ptC.getX(), (int)ptC.getY());
        ((Graphics2D)gr).rotate(Math.toRadians(angles[2]));
        int length = (int)Math.sqrt(Math.pow(Math.abs(ptB.getX() - ptC.getX()), 2) + 
                Math.pow(Math.abs(ptB.getY() - ptC.getY()), 2));
        Rectangle r = new Rectangle(-length, -20, length, 12);
        drawRotatedStringInRectangle(gr, r, strA + a.getText());
        ((Graphics2D)gr).setTransform(at);
        
        gr.translate((int)ptB.getX(), (int)ptB.getY());
        ((Graphics2D)gr).rotate(-Math.toRadians(angles[0]));
        length = (int)Math.sqrt(Math.pow(Math.abs(ptA.getX() - ptB.getX()), 2) + 
                Math.pow(Math.abs(ptA.getY() - ptB.getY()), 2));
        r = new Rectangle(-length, -15, length, 12);
        drawRotatedStringInRectangle(gr, r, strC + c.getText());
        ((Graphics2D)gr).setTransform(at);
        
        r = new Rectangle((int)ptA.getX(), (int)ptA.getY() + 10, (int)(ptC.getX() - ptA.getX()), 12);
        drawRotatedStringInRectangle(gr, r, strB + b.getText());

    }
    
    public static void drawAnglesLabels(Graphics gr, String A, String B, String C, boolean noLabel, Point... points)
{
		//int startAngle = fixDecimal(FindAngle(points[0], points[1]));
		int sweepAngle = fixDecimal(FindAngle(points[0], points[2]) - FindAngle(points[0], points[1]));
		if (sweepAngle < 0)
		{
			sweepAngle = 360 + sweepAngle;
		}

                int midX = (int)(points[1].getX() + points[2].getX()) / 2;
                int midY = (int)(points[1].getY( )+ points[2].getY()) / 2;
                Point intersectionPoint = new Point(midX, midY);
		Point ip = MaxDistanceEndPoint(points[0], intersectionPoint, 50);
		Point ep = MaxDistanceEndPoint(points[0], intersectionPoint, -12);
                
                boolean isRight = A.equals("90");

		if (isRight)
		{
                    Point pt1 = points[0];
                    Point pt2 = new Point(((int)points[0].getX() + 20), ((int)points[0].getY() -20));

                    int x1 = (int)((pt1.getX() + pt2.getX()) / 2.0 + (pt1.getY() - pt2.getY()) / 2.0);
                    int y1 = (int)((pt1.getY() + pt2.getY()) / 2.0 + (pt2.getX() - pt1.getX()) / 2.0);
                    int x2 = (int)((pt1.getX() + pt2.getX()) / 2.0 - (pt1.getY() - pt2.getY()) / 2.0);
                    int y2 = (int)((pt1.getY() + pt2.getY()) / 2.0 - (pt2.getX() - pt1.getX()) / 2.0);

                    gr.drawLine(x1, y1, (int)pt2.getX(), (int)pt2.getY());
                    gr.drawLine((int)pt2.getX(), (int)pt2.getY(), x2, y2);
			
		}
		else
		{
                    gr.drawArc(((int)points[0].getX() - 20), ((int)points[0].getY() - 20), 40, 40, 0, sweepAngle);
                    if(!noLabel) {
                        centreTextOnPoint(gr, A + "º", ip);
                    }
                }

                if(!noLabel) {
                    centreTextOnPoint(gr, "A", ep);
                }              

		//startAngle = fixDecimal(FindAngle(points[1], points[2]));
		sweepAngle = fixDecimal(FindAngle(points[1], points[0]) - FindAngle(points[1], points[2]));
		if (sweepAngle < 0)
		{
			sweepAngle = 360 + sweepAngle;
		}

		midX = (int)(points[2].getX() + points[0].getX()) / 2;
                midY = (int)(points[2].getY( )+ points[0].getY()) / 2;
                intersectionPoint = new Point(midX, midY);
		ip = MaxDistanceEndPoint(points[1], intersectionPoint, 50);
		ep = MaxDistanceEndPoint(points[1], intersectionPoint, -12);
                
                isRight = B.equals("90");

		if (isRight)
		{
			ArrayList<line> parallelLines = new ArrayList<line>();

			float l = (float)Math.sqrt((points[1].getX() - points[0].getX()) * (points[1].getX() - points[0].getX()) + (points[1].getY() - points[0].getY()) * (points[1].getY() - points[0].getY()));
			parallelLines.add(new line((int)(points[1].getX() + 20 * (points[0].getY() - points[1].getY()) / l), (int)(points[0].getX() + 20 * (points[0].getY() - points[1].getY()) / l), (int)(points[1].getY() + 20 * (points[1].getX() - points[0].getX()) / l), (int)(points[0].getY() + 20 * (points[1].getX() - points[0].getX()) / l)));

			l = (float)Math.sqrt((points[1].getX() - points[2].getX()) * (points[1].getX() - points[2].getX()) + (points[1].getY() - points[2].getY()) * (points[1].getY() - points[2].getY()));
			parallelLines.add(new line((int)(points[1].getX() - 20 * (points[2].getY() - points[1].getY()) / l), (int)(points[2].getX() - 20 * (points[2].getY() - points[1].getY()) / l), (int)(points[1].getY() - 20 * (points[1].getX() - points[2].getX()) / l), (int)(points[2].getY() - 20 * (points[1].getX() - points[2].getX()) / l)));

			Point pt2 = MaxDistanceEndPoint(new Point(parallelLines.get(0).getX1(), parallelLines.get(0).getY1()), new Point(parallelLines.get(0).getX2(), parallelLines.get(0).getY2()), 20);

                        gr.drawLine(parallelLines.get(0).getX1(), parallelLines.get(0).getY1(), (int)pt2.getX(), (int)pt2.getY());
                        gr.drawLine((int)pt2.getX(), (int)pt2.getY(), parallelLines.get(1).getX1(), parallelLines.get(1).getY1());
		}
		else
		{
                    gr.drawArc(((int)points[1].getX() - 20), ((int)points[1].getY() - 20), 40, 40, 180 + (int)Float.parseFloat(A), sweepAngle);
                    if(!noLabel) {
                        centreTextOnPoint(gr, B + "º", ip);	
                    }
		}
                if(!noLabel) {
                    centreTextOnPoint(gr, "B", ep);
                }
                
		//startAngle = fixDecimal(FindAngle(points[2], points[0]));
		sweepAngle = fixDecimal(FindAngle(points[2], points[1]) - FindAngle(points[2], points[0]));
		if (sweepAngle < 0)
		{
			sweepAngle = 360 + sweepAngle;
		}

		midX = (int)(points[0].getX() + points[1].getX()) / 2;
                midY = (int)(points[0].getY() + points[1].getY()) / 2;
                intersectionPoint = new Point(midX, midY);
		ip = MaxDistanceEndPoint(points[2], intersectionPoint, 50);
		ep = MaxDistanceEndPoint(points[2], intersectionPoint, -12);
                
                isRight = C.equals("90");

		if (isRight)
		{
                    Point pt1 = points[2];
                    Point pt2 = new Point(((int)points[2].getX() - 20), ((int)points[2].getY() - 20));

                    int x1 = (int)((pt1.getX() + pt2.getX()) / 2.0 + (pt1.getY() - pt2.getY()) / 2.0);
                    int y1 = (int)((pt1.getY() + pt2.getY()) / 2.0 + (pt2.getX() - pt1.getX()) / 2.0);
                    int x2 = (int)((pt1.getX() + pt2.getX()) / 2.0 - (pt1.getY() - pt2.getY()) / 2.0);
                    int y2 = (int)((pt1.getY() + pt2.getY()) / 2.0 - (pt2.getX() - pt1.getX()) / 2.0);

                    gr.drawLine(x1, y1, (int)pt2.getX(), (int)pt2.getY());
                    gr.drawLine((int)pt2.getX(), (int)pt2.getY(), x2, y2);
		}
		else
		{
                    gr.drawArc(((int)points[2].getX() - 20), ((int)points[2].getY() - 20), 40, 40, 180 - sweepAngle, sweepAngle);
                    if(!noLabel) {
                        centreTextOnPoint(gr, C + "º", ip);	
                    }
		}
                
                if(!noLabel) {
                    centreTextOnPoint(gr, "C", ep);
                }

	}

    public static BufferedImage drawBaseImage(JTextField[] tbs, JLabel[] lbls)
    {
        int w = 285;
        int h = 130;
        
        BufferedImage img = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
        Graphics gr = img.createGraphics();
                
        RenderingHints rh = new RenderingHints(
                 RenderingHints.KEY_TEXT_ANTIALIASING,
                 RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        ((Graphics2D)gr).setRenderingHints(rh);

        int[] xPositions = {17, 107, 197};
        int[] yPositions = {20, 80};

        

        gr.setColor(Color.white);
        gr.fillRect(0, 0, w, h);
        gr.setColor(Color.black);

        for (int x = 0; x <= 2; x++)
        {
            gr.drawString(lbls[x].getText(), xPositions[x], yPositions[0]);
            if (lbls.length > 3)
            {
                gr.drawString(lbls[x + 3].getText(), xPositions[x], yPositions[1]); //here
            }
        }

        return img;
    }

    public static void drawValuesImage(BufferedImage baseImage, Path filePath, JTextField[] tbs, JLabel[] lbls)
    {
        BufferedImage img = new BufferedImage(baseImage.getWidth(), baseImage.getHeight(), BufferedImage.TYPE_INT_ARGB);
        Graphics gr = img.createGraphics();
        
        RenderingHints rh = new RenderingHints(
                 RenderingHints.KEY_TEXT_ANTIALIASING,
                 RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        ((Graphics2D)gr).setRenderingHints(rh);
        
        gr.drawImage(baseImage, 0, 0, null);

        int[] xPositions = {17, 107, 197, 17, 107, 197};
        int[] yPositions = {27, 87};
        for (int x = 0; x <  tbs.length; x++)
        {
            Color c = (Color)((tbs[x].isEnabled()) ? Color.white : Color.lightGray);
            Rectangle r = new Rectangle(xPositions[x], yPositions[(x > 2 ? 1 : 0)], 75, 30);
            //r.Offset(new Point(-lbls[0].Left + 10, -lbls[0].Top + 10));
            gr.setColor(c);
            gr.fillRect((int)r.getX(), (int)r.getY(), (int)r.getWidth(), (int)r.getHeight());
            gr.setColor(Color.black);
            gr.drawRect((int)r.getX(), (int)r.getY(), (int)r.getWidth(), (int)r.getHeight());
            drawStringInRectangle(gr, r, tbs[x].getText());
        }

        try
        {
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            ImageIO.write(img, "png", os);
            InputStream is = new ByteArrayInputStream(os.toByteArray(), 0, os.size());
            Files.copy(is, filePath, StandardCopyOption.REPLACE_EXISTING);
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
		
    }

    private static void drawStringInRectangle(Graphics g, Rectangle r, String s) {
        Graphics2D g2d = (Graphics2D)g;
        FontMetrics fm = g2d.getFontMetrics();
        Rectangle2D r2 = fm.getStringBounds(s, g2d);
        int x = ((int)r.getWidth() - (int)r2.getWidth()) / 2;
        int y = ((int)r.getHeight() - (int)r2.getHeight()) / 2 + fm.getAscent();
        g.drawString(s, (int)r.getX() + x, (int)r.getY() + y);
    }
    
    public static void centreTextOnPoint(Graphics g, String s, Point p)
    {
        Rectangle r = new Rectangle((int)p.getX() - 10, (int)p.getY() - 10, 20, 20);
        drawStringInRectangle(g, r, s);
    }
    
    private static void drawRotatedStringInRectangle(Graphics g, Rectangle r, String s) {
        Graphics2D g2d = (Graphics2D)g;
        FontMetrics fm = g2d.getFontMetrics();
        Rectangle2D r2 = fm.getStringBounds(s, g2d);
        int x = ((int)r.getWidth() - (int)r2.getWidth()) / 2;
        int y = ((int)r.getHeight() - (int)r2.getHeight()) / 2 + fm.getAscent();
        g.drawString(s, (int)r.getX() + x, (int)r.getY() + y);
    }
            
}