/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package html.triangle;

import java.awt.Point;
import html.shared.points.*;

/**
 *
 * @author Paul Long
 * 
 * Triangle class. Used for plotting triangle points.
 * Only used for non right angled triangles with the 
 * sine and cosine rule solvers
 */
public class Triangle
{
    
	private PointF A;
	private PointF B;
	private PointF C;
        
        /**
     * @return Point A
     */
    public Point getA()
    {
        return new Point((int)this.A.getX(), (int)this.A.getY());
    }

    /**
     * @return Point B
     */
    public Point getB()
    {
        return new Point((int)this.B.getX(), (int)this.B.getY());
    }

    /**
     * @return Point C
     */
    public Point getC()
    {
        return new Point((int)this.C.getX(), (int)this.C.getY());
    }
    
    /** 
    
     @param sideA The length of the known side BC
     @param sideB The length of the known side AC
     @param sideC The length of the known side AB
     @param angleA The internal angle in Radians at vertex A
     @param angleB The internal angle in Radians at vertex B
     @param angleC The internal angle in Radians at vertex C

    */
    public Triangle(float sideA, float sideB, float sideC, float angleA, float angleB, float angleC)
    {
        
        float bX = (float)(Math.cos(angleA) * sideC);
        float bY = (float)(Math.sin(angleA) * sideC);
        
        this.A = new PointF(0, 0);
        this.C = new PointF(this.A.getX() + sideB, this.A.getY());
        this.B = new PointF(bX, bY);

        if (bX < 0)
        {
            this.A = new PointF((this.A.getX() + Math.abs(bX)), this.A.getY());
            this.B = new PointF((this.B.getX() + Math.abs(bX)), this.B.getY());
            this.C = new PointF((this.C.getX() + Math.abs(bX)), this.C.getY());            
        }

    }

    public final void ScaleToFit(float maxWidthOrHeight)
    {
        float[] xCoords = {(float)this.A.getX(), (float)this.B.getX(), (float)this.C.getX()};
        float min = 10000;
        float max = -1;

        for(int x = 0; x < 3; x++) {
            if(xCoords[x] < min) {min = xCoords[x];}
            if(xCoords[x] > max) {max = xCoords[x];}
        }
        
        float OverallWidth = (float)(max - min);
        
        float[] yCoords = {(float)this.A.getY(), (float)this.B.getY(), (float)this.C.getY()};
        min = 10000;
        max = -1;

        for(int x = 0; x < 3; x++) {
            if(yCoords[x] < min) {min = yCoords[x];}
            if(yCoords[x] > max) {max = yCoords[x];}
        }        
        
        float OverallHeight = (float)(max - min);
        
        float scaleFactor = (float)(OverallWidth > OverallHeight ? maxWidthOrHeight / OverallWidth : maxWidthOrHeight / OverallHeight);
        
        Scale(scaleFactor);
        centreTriangle(25, 300);
    }

    private void Scale(float scaleFactor)
    {
       this.A = ScalePoint(this.A, scaleFactor);
       this.B = ScalePoint(this.B, scaleFactor);
       this.C = ScalePoint(this.C, scaleFactor);
    }

    private PointF ScalePoint(PointF p, float factor)
    {
        return new PointF((p.getX() * factor), (p.getY() * factor));
    }

    private void centreTriangle(int border, int displaySize)
    {
        int y1 = (int)this.A.getY();
        int y2 = (int)this.B.getY();
        if(y2 > y1) {y2 -= ((y2 - y1) * 2);}
        this.B = new PointF(this.B.getX(), y2);
        
        PointF[] pts = new PointF[] {this.A, this.B, this.C};
        float min = 10000;
        float max = -1;

        for(int x = 0; x < 3; x++) {
            if(pts[x].getX() < min) {min = (float)pts[x].getX();}
            if(pts[x].getY() > max) {max = (float)pts[x].getY();}
        }

        int offset_X = (int)(min - border);
        int offset_Y = (int)(max - (displaySize - border));
        this.A = new PointF((this.A.getX() - offset_X), (this.A.getY() - offset_Y));
        this.B = new PointF((this.B.getX() - offset_X), (this.B.getY() - offset_Y));
        this.C = new PointF((this.C.getX() - offset_X), (this.C.getY() - offset_Y));
    }

}

